<?php
namespace Teknisa\EST\Services;

use Teknisa\EST\Factories\Ger01700UnidadeFactory;
use Zeedhi\Framework\Util\Functions;

class Ger01700UnidadeService {
    protected $factory;
    protected $cripto;

    /**
     * Construtor do Service.
     *
     * @param Ger01700UnidadeFactory $factory
     */
    public function __construct($cripto, Ger01700UnidadeFactory $factory) {
        $this->factory = $factory;
        $this->cripto  = $cripto;
    }

    /**
     * Procedimento que efetua a atualização dos produtos
     * para uma unidade de acordo com os dados informados
     *
     * @throws \Exception
     *
     * @param string $CDFILIAL
     * @param int $CDNVPRODUTO
     * @param int $NRORG
     */
    public function atualizaProduto($CDFILIAL, $CDNVPRODUTO, $NRORG) {
        $factoryReturn = $this->factory->getIDCTRLESTQ($CDFILIAL, $NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMFILIAL na unidade ".$CDFILIAL." da organização ".$NRORG."!", 1);

        $IDCTRLESTQ = $factoryReturn[0]["IDCTRLESTQ"];

        $factoryReturn = $this->factory->getIDUTILLOTE($CDFILIAL, $NRORG);
        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMFILIAL na unidade ".$CDFILIAL." da organização ".$NRORG."!", 1);

        $IDUTILLOTE = $factoryReturn[0]["IDUTILLOTE"];

        $this->factory->executeAtualizaProduto($CDFILIAL, $CDNVPRODUTO, $IDCTRLESTQ, $IDUTILLOTE);
    }

    /**
     * Função que retorna as validações de endereço de
     * uma unidade.
     *
     * @throws \Exception
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return array
     */
    public function validateEndefili($CDFILIAL, $NRORG) {
        $addressTypes = array('C', 'E', 'P', 'T', 'X');

        $validation = array();

        if($CDFILIAL != 'IGNORE') {
            $factoryReturn = $this->factory->getValidationForEndefili($CDFILIAL, $NRORG);
            if(count($factoryReturn) != 1)
                throw new \Exception("Erro Catastrófico! Erro na validação do endereço da unidade ".$CDFILIAL." da organização ".$NRORG."!", 1);

            $validation['CDFILIAL']     = $factoryReturn[0]['CDFILIAL'];
            $validation['ALLTYPES']     = $addressTypes;
            $validation['USEDTYPES']    = explode('.', $factoryReturn[0]['CDROWS']);
            $validation['FREETYPES']    = array_values(array_diff($validation['ALLTYPES'], $validation['USEDTYPES']));
            $validation['MAYINSERT']    = count($validation['FREETYPES']) > 0;
        }

        $factoryReturn = $this->factory->getValidationForEndefiliFromParamgeral($NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMGERAL!", 1);

        $validation['REQENDFIELDS'] = ($factoryReturn[0]['IDUTENDMODFISC'] == 'S');

        return $validation;
    }

    /**
     * Função que cria campos conforme necessário para a
     * inserção/edição de Endefili.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateEndefiliRow($row) {
        #concats address description
        $row['DSENDEFILI'] = ((Functions::arrayKeyExists('DSENDEFILISEP', $row) && Functions::arrayKeyExists('NRENDEFILISEP', $row))? $row['DSENDEFILISEP'].', '.$row['NRENDEFILISEP']: '');

        #concats numbers
        $row['NRFAXFILI'] = ((Functions::arrayKeyExists('NRDDDFAXFILI', $row) && Functions::arrayKeyExists('NRFAXFILISEP', $row))? $row['NRDDDFAXFILI'].$row['NRFAXFILISEP']: '');
        $row['NRTELEFILI'] = ((Functions::arrayKeyExists('NRDDDTELEFILI', $row) && Functions::arrayKeyExists('NRTELEFILISEP', $row))? $row['NRDDDTELEFILI'].$row['NRTELEFILISEP']: '');
        $row['NRTELEXFILI'] = ((Functions::arrayKeyExists('NRDDDTELEXFILI', $row) && Functions::arrayKeyExists('NRTELEXFILISEP', $row))? $row['NRDDDTELEXFILI'].$row['NRTELEXFILISEP']: '');

        #encrypts password
        $row['CDSENHEMAILF'] = (Functions::arrayKeyExists('CDSENHEMAILF', $row))? $this->cripto->cripto($row['CDSENHEMAILF'], 2) : null;

        #reduces neighborhood name size, if needed
        $row['NMBAIRFILI'] = (Functions::arrayKeyExists('NMBAIRFILI', $row))? substr($row['NMBAIRFILI'], 0, 30) : null;

        return($row);
    }

    /**
     * Função que verifica a tabela PARAMGERAL para
     * checar a parametrização de uso de parâmetros de
     * unidade padrão.
     *
     * @throws \Exception
     *
     * @return array
     */
    public function validateParamfilipadr($NRORG) {
        $factoryReturn = $this->factory->getValidationForParamfilipadr($NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na tabela PARAMGERAL!", 1);

        $validation                 = array();
        $validation['NRORG']        = $factoryReturn[0]['NRORG'];
        $validation['SHOWPARAMTAB'] = ($factoryReturn[0]['IDSHOWPARAMS'] == 'S');

        return $validation;
    }

    /**
     * Função que cria campos conforme necessário para a
     * inserção de Unidade.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateUnidadeNewRow($row) {
        try{
            if(!Functions::arrayKeyExists('IDALMOXINVAENTR', $row))
                $row['IDALMOXINVAENTR'] = 'N';
    
            if(!Functions::arrayKeyExists('IDCTRLESTQ', $row))
                $row['IDCTRLESTQ'] = 'N';
    
            return($row);
        } catch (\Exception $e){
            
        }
    }


    /**
     * Função que retorna as validações da primary key de
     * uma unidade.
     *
     * @throws \Exception
     *
     * @param string $CDFILIAL
     * @param int $NRORG
     *
     * @return array
     */
    public function validateUnidadePK($CDFILIAL, $NRORG) {
        $factoryReturn = $this->factory->getValidationForUnidadePK($CDFILIAL, $NRORG);

        if(count($factoryReturn) != 1)
            throw new \Exception("Erro Catastrófico! Erro na requisição de validação de código de unidade!", 1);

        $validation          = array();
        $validation['NRORG'] = $factoryReturn[0]['NRORG'];
        $validation['CODE']  = $factoryReturn[0]['CDFILIAL'];
        $validation['VALID'] = ($factoryReturn[0]['IDFREECODE'] == 'S');

        return $validation;
    }

    /**
     * Função que cria campos conforme necessário para a
     * inserção/edição de Unidade.
     *
     * @param array $row
     *
     * @return array
     */
    public function validateUnidadeRow($row) {
        if(!Functions::arrayKeyExists('NMRAZSOCFILI', $row) || !$row['NMRAZSOCFILI'])
            $row['NMRAZSOCFILI'] = $row['NMFILIAL'];

        if(!Functions::arrayKeyExists('CDALMOXARIFE', $row))
            $row['CDALMOXARIFE'] = ' ';
        if(!Functions::arrayKeyExists('DSALMOXARIFE', $row))
            $row['DSALMOXARIFE'] = ' ';

        if(!Functions::arrayKeyExists('CDLOCALESTOQ', $row))
            $row['CDLOCALESTOQ'] = ' ';
        if(!Functions::arrayKeyExists('DSLOCALESTOQ', $row))
            $row['DSLOCALESTOQ'] = ' ';

        return($row);
    }
}