<?php

/**
 * GepProdutosoliLogic.
 *
 * @package Logic
 * @author  erikasouza
 */
class GepProdutosoliLogic {


    private $_GepProdutosoliEntity;


    /**
     * Construtor da classe.
     */
    public function __construct(){
        $this->_GepProdutosoliEntity = new GepProdutosoliEntity();
    }


    /**    ok
     * Retorna produtos de uma determianda solicitacao.
     *
     * @param integer $nrorg         N?mero da Organizacao.
     * @param integer $nrsolicitacao N?mero da Solicitacao.
     *
     * @return Ambigous <mixed, NULL>|boolean
     */
    public static function retornaProdutoSoli($nrorg, $nrsolicitacao){
        try {
            $resultado = GepProdutosoliEntity::retornaProdutoSoli($nrorg, $nrsolicitacao);
            return $resultado;
        } catch (Exception $e) {
            switch ($e->getCode()){
            case GepProdutosoliEntity::SEM_PRODUTOS:
                return GepProdutosoliEntity::SEM_PRODUTOS;
            default:
                tk_warning('Erro desconhecido: '.$e->getMessage());
                return false;
            }
        }

    }


    /**     ok
     * Retorna Lista de Produtos de uma solicitacao.
     *
     * @param integer $nrorg         Numero da Organizacao.
     * @param integer $nrsolicitacao Solicitacao.
     *
     * @return Ambigous <boolean, NULL, TkModelEntity>
     */
    public function consultaListaProdutoSoli($nrorg, $nrsolicitacao){
        try {
            $resultado = $this->_GepProdutosoliEntity->consultaListaProdutoSoli($nrorg, $nrsolicitacao);
            return $resultado;
        } catch(Exception $e) {
            switch ($e->getCode())
            {
            case GepProdutosoliEntity::SEM_PRODUTOS:
                return 'SEM_PRODUTOS';
            default:
                tk_message('Erro desconhecido: '.$e->getMessage());
                return 'ERRO';
            }
        }

    }


    /**
     * Insere um novo produto com uma solicita??o determinada.
     *
     * @param integer $solicitacao N?mero da solicita??o.
     * @param integer $produto     C?digo do produto.
     * @param integer $operador    C?digo do Operador.
     * @param date    $data        Data da inser??o.
     * @param integer $nrorg       N?mero da Organiza??o.
     *
     * @return boolean
     */
    public function insereProdutoSoli(
        $solicitacao, $produto, $operador, date $data, $nrorg
    ){
        $codigo = $this->_GepProdutosoliEntity->novoCodigoGepProdutoSoli($nrorg);
        $insere = $this->_GepProdutosoliEntity->insereProdutoSoli(
            $codigo, $solicitacao, $produto, $operador, $data, $nrorg
        );
        return $insere;
    }


    /**      ok
     * Gera novo codigo do produto da solicitacao.
     *
     * @param integer $nrorg N?mero da Organizacao.
     *
     * @return Ambigous <number, number>
     */
    public static function novoCodigoGepProdutoSoli($nrorg){
        $codigo = GepProdutosoliEntity::novoCodigoGepProdutoSoli($nrorg);
        return $codigo;
    }


    /**
     * Atualiza o produto de uma determinada solicita??o.
     *
     * @param integer $nrorg       N?mero da Organiza??o.
     * @param integer $solicitacao N?mero da solicita??o.
     * @param integer $produto     C?digo do novo produto.
     * @param integer $operador    C?digo do operador de altera??o.
     * @param date    $data        Data da altera??o.
     *
     * @return boolean
     */
    public function atualizaProdutoSoli(
    $nrorg, $solicitacao, $produto, $operador, date $data
    ){
        $atualizacao = $this->_GepProdutosoliEntity->atualizaProdutoSoli(
            $nrorg, $solicitacao, $produto, $operador, $data
        );
        return $atualizacao;
    }


    /**
     * Exclui todos os produtos de uma determinada solicitacao.
     *
     * @param integer $solicitacao N?mero da Solicita??o.
     * @param integer $nrorg       N?mero da Organiza??o.
     *
     * @return boolean
     */
    public function excluiProdutoSoli($solicitacao, $nrorg){
        $exclusao = $this->_GepProdutosoliEntity->excluiProdutoSoli($solicitacao, $nrorg);
        return $exclusao;
    }


    /**
     * Salva o produto de uma determianda solicitacao.
     *
     * @param integer $nrorg       Numero da organizacao.
     * @param integer $solicitacao Numero da solicitacao.
     * @param integer $produto     Codigo do produto.
     * @param integer $operador    Codigo do operador.
     * @param date    $data        Data.
     *
     * @return boolean
     */
    public function salvaProdutoSoli($nrorg, $solicitacao, $produto, $operador, date $data){
        $dados_produtosoli = $this->retornaProdutoSoli($nrorg, $solicitacao);
        if ($dados_produtosoli === false) {
            $insere_produtosoli = $this->insereProdutoSoli(
                $solicitacao, $produto, $operador, $data, $nrorg
            );
        } else {
            $atualiza_produtosoli = $this->atualizaProdutoSoli(
                $nrorg, $solicitacao, $produto, $operador, $data
            );
        }

        return true;
    }


}